# Please execute this program with the command 'python savefield.py' or 'python3 savefield.py'.
# The programa will generate a TeX file (field.tex).
# The TeX file can be compiled into PDF using the program pdflatex.

from numpy import *
from math import *

texbegin =\
'\\documentclass[12pt]{standalone}\n' +\
'\\usepackage{tikz}\n' +\
'\\begin{document}\n' +\
'\\begin{tikzpicture}\n'

texend =\
'\\end{tikzpicture}\n' +\
'\\end{document}\n'

def tikzmarks(t0, t1, x0, x1, dtmarks, dxmarks, tscale, xscale, tborder, xborder):
   result = ''
   jmin = math.ceil((t0 - 0.5 * tborder) / dtmarks)
   jmax = math.floor((t1 + 0.5 * tborder) / dtmarks)
   kmin = math.ceil((x0 - 0.5 * xborder) / dxmarks)
   kmax = math.floor((x1 + 0.5 * xborder) / dxmarks)
   (tgmin, tgmax) = ((t0 - tborder) * tscale, (t1 + tborder) * tscale)
   (xgmin, xgmax) = ((x0 - xborder) * xscale, (x1 + xborder) * xscale)
   result += '\\draw ({0:.4f}, {1:.4f}) rectangle ({2:.4f}, {3:.4f});'.format(tgmin, xgmin, tgmax, xgmax)
   for t in [j * tscale for j in range(jmin, jmax + 1)]:
      tstring = str(int(t)) if (t == round(t)) else str(t)
      result += '\\draw ({0:.4f}, {1:.4f}) -- ({0:.4f}, {2:0.4f});'.format(t * tscale, xgmin - 0.1, xgmin)
      result += '\\node[below] at ({0:.4f}, {1:.4f}) {{\\tiny ${2}$}};'.format(t * tscale, xgmin - 0.1, tstring)
   for x in [k * xscale for k in range(kmin, kmax + 1)]:
      xstring = str(int(x)) if (x == round(x)) else str(x)
      result += '\\draw ({1:.4f}, {0:.4f}) -- ({2:.4f}, {0:0.4f});'.format(x * xscale, tgmin - 0.1, tgmin)
      result += '\\node[left] at ({1:.4f}, {0:.4f}) {{\\tiny ${2}$}};'.format(x * xscale, tgmin - 0.1, xstring)
   return result

def tikzfield(f, t0, t1, x0, x1, nt, nx, tscale, xscale):
   result = ''
   h = (t1 - t0) * 1.0 / nt
   r = h / 4
   for t in linspace(t0, t1, nt + 1):
      for x in linspace(x0, x1, nx + 1):
         p = f(t, x)
         l = sqrt(1.0 + p * p)
         (dt, dx) = (r / l, r * p / l)
         (tg0, tg1) = ((t - dt) * tscale, (t + dt) * tscale)
         (xg0, xg1) = ((x - dx) * xscale, (x + dx) * xscale)
         result += '\\draw ({0:.4f}, {1:.4f}) -- ({2:.4f}, {3:.4f});\n'.format(tg0, xg0, tg1, xg1)
   return result

def tikzfunction(f, t0, t1, nt, tscale, xscale):
   result = '\\draw[blue,thick] plot[smooth] coordinates {\n'
   for t in linspace(t0, t1, nt + 1):
      x = f(t)
      (tg, xg) = (t * tscale, x * tscale)
      result += '({0:.4f}, {1:.4f})\n'.format(tg, xg)
   result += '};'
   return result

def savefield(filename, field, sol, t0, t1, x0, x1, dtmarks, dxmarks, nt, nx, tscale, xscale, tborder, xborder):
   s = texbegin
   s += tikzmarks(t0, t1, x0, x1, dtmarks, dxmarks, tscale, xscale, tborder, xborder)
   s += tikzfield(field, t0, t1, x0, x1, nt, nx, tscale, xscale)
   s += tikzfunction(sol, t0, t1, 5 * nt, tscale, xscale)
   s += texend
   with open(filename, "w") as textfile:
      textfile.write(s)

def f1(t, x):
   return 2 * x / (3 + t)

def solution1(t):
   return ((3.0 + t) ** 2) / 9;

savefield('field.tex', f1, solution1, -2.0, 2.0, -2.0, 4.0, 1, 1, 10, 20, 1.0, 1.0, 0.15, 0.15)

